{% extends "modals/base_modal.html" %}
{% load i18n %}
{% block modal_title %}{% trans "Resubmission Complete" %}{% endblock modal_title %}
{% block modal_body %}{% trans "Your task has been successfully resubmitted." %}{% endblock modal_body %}
{% block modal_buttons %}
<a href="#" id="new-task-url"><button class="btn btn-primary">{% trans "View Task" %}</button></a>
{% endblock modal_buttons %}
